// Program 8.18 i boken.
package spill;    // Fil: spill/Spillebrett.java

public class Spillebrett {
  // Konstanter:                                                      // (1)
  public static final int RAD_ANTALL = 6;
  public static final int KOLONNE_ANTALL = 7;
  public static final int RUTE_ANTALL = RAD_ANTALL * KOLONNE_ANTALL;
  private static final int INGEN_LEDIG_RAD = -1;

  // Felter som ikke kan endres:                                      // (2)
  final IGrensesnitt grensesnitt;
  final IBrikkeIdentitet[] brikkeIdentiteter;
  final IBrikkeIdentitet tomRute;

  int trekkTeller;                                                    // (3)

  // private felter som gies adgang til gjennom metoder:              // (4)
  private IBrikkeIdentitet[][] ruteInnhold =
      new IBrikkeIdentitet[KOLONNE_ANTALL][RAD_ANTALL];
  private IBrikkeIdentitet vinner;

  // public konstruktrer og metoder:                                 // (5)

  public Spillebrett(IBrikkeIdentitet[] identiteter, IBrikkeIdentitet tomRute) {
    this(identiteter, tomRute, null);
  }

  public Spillebrett(IBrikkeIdentitet[] identiteter, IBrikkeIdentitet tomRute,
	      IGrensesnitt grensesnitt)
  {
    this.tomRute = tomRute;
    this.grensesnitt = grensesnitt;

    assert identiteter.length > 0;
    brikkeIdentiteter = identiteter;
    nullstill();

    if (grensesnitt != null)
      grensesnitt.settBrett(this);
  }

  public void nullstill() {
    vinner = null;
    trekkTeller = 0;
    for (IBrikkeIdentitet[] kolonne : ruteInnhold) {
      for (int radIndeks = 0; radIndeks < RAD_ANTALL; ++radIndeks) {
	kolonne[radIndeks] = tomRute;
      }
    }
  }

  public IBrikkeIdentitet brikkeVed(int kolonne, int rad) {
    return ruteInnhold[kolonne][rad];
  }

  public boolean harVinner() {
    return vinner != null;
  }

  public boolean erSpillFerdig() {
    return (trekkTeller >= RUTE_ANTALL) || harVinner();
  }

  public void slippBrikkeNedKolonne(IBrikkeIdentitet brikke, int kolonne) {
    assert !erSpillFerdig();
    int rad = finnNedersteLedigeRad(kolonne);
    ruteInnhold[kolonne][rad] = brikke;
    if (lengsteSekvensSomInneholderRute(kolonne, rad) >= 4)
      vinner = brikke;
    ++trekkTeller;
    giRapportTilGrensesnitt();
  }

  public boolean erKolonneFull(int kolonne) {
    IBrikkeIdentitet[] kolonneInnhold = ruteInnhold[kolonne];
    int indeksForversteRute = kolonneInnhold.length - 1;
    IBrikkeIdentitet versteRute = kolonneInnhold[indeksForversteRute];
    return versteRute != tomRute;
  }

  public boolean erGyldigKolonneValg(int kolonne) {
    return gyldigKolonneIndeks(kolonne)
        && !erKolonneFull(kolonne);
  }

  public IBrikkeIdentitet nesteBrikke() {
    return brikkeIdentiteter[trekkTeller % brikkeIdentiteter.length];
  }

  protected void giRapportTilGrensesnitt() {                          // (6)
    if (grensesnitt == null)
      return;

    grensesnitt.brikkeBleSluppet();
    if (!erSpillFerdig()) {
      return;
    }

    if (harVinner()) {
      grensesnitt.spillerHarVunnet(vinner);
    } else {
      grensesnitt.spillEndteUavgjort();
    }
  }

  // private konstruktrer og metoder:                                // (7)

  private int finnNedersteLedigeRad(int kolonne) {
    IBrikkeIdentitet[] kolonneInnhold = ruteInnhold[kolonne];
    for (int rad = 0; rad < kolonneInnhold.length; ++rad) {
      if (kolonneInnhold[rad] == tomRute)
        return rad;
    }
    return INGEN_LEDIG_RAD;
  }

  private int lengsteSekvensSomInneholderRute(int kolonne, int rad) {
    int strsteLengde = 1;
    // Let etter sekvenser av like brikker i alle retninger.
    for (Retning retning : Retning.RETNINGER) {
      int antallBrikkerPLinje = 1 + antallLikeRuterIRetning(kolonne, rad, retning)
          + antallLikeRuterIRetning(kolonne, rad, retning.motsatt());
      strsteLengde = Math.max(strsteLengde, antallBrikkerPLinje);
    }
    return strsteLengde;
  }

  private int antallLikeRuterIRetning(int kolonne, int rad, Retning retning) {
    IBrikkeIdentitet brikke = brikkeVed(kolonne, rad);
    int antall = 0;
    while (true) {
      kolonne += retning.kolonneSkritt;
      rad += retning.radSkritt;
      if (!posisjonInneholderBrikke(kolonne, rad, brikke))
        break;
      ++antall;
    }
    return antall;
  }

  private boolean gyldigKolonneIndeks(int kolonne) {
    return (kolonne >= 0) && (kolonne < KOLONNE_ANTALL);
  }

  private boolean gyldigRadIndeks(int rad) {
    return (rad >= 0) && (rad < RAD_ANTALL);
  }

  private boolean posisjonInneholderBrikke(int kolonne, int rad,
					   IBrikkeIdentitet brikke)
  {
    return gyldigKolonneIndeks(kolonne)
        && gyldigRadIndeks(rad)
        && (brikkeVed(kolonne, rad) == brikke);
  }
}
