package spill.test;    // Fil: spill/test/TestSpillebrett.java
import junit.framework.TestCase;
import spill.Spillebrett;

public class TestSpillebrett extends TestCase {
  public void testTomRute() {
    Spillebrett brett = SpillSimulator.byggSpillebrett("");
    // Nederste rad har indeks 0.
    int rad = 0;
    // Kolonne lengst til venstre har ogs indeks 0
    int kolonne = 0;
    assertEquals("Tom", SpillSimulator.TOM_RUTE, brett.brikkeVed(kolonne, rad));
  }

  public void testSlippBrikke() {
    Spillebrett brett = SpillSimulator.byggSpillebrett("02");
    /* De to nederste radene skal n se slik ut:

       1 | . . . . . . . |
       0 | X . O . . . . |
         -----------------
        /  0 1 2 3 4 5 6  \
    */
    assertEquals("Rute 0,0", SpillSimulator.X_BRIKKE, brett.brikkeVed(0, 0));
    assertEquals("Rute 1,0", SpillSimulator.TOM_RUTE, brett.brikkeVed(1, 0));
    assertEquals("Rute 2,0", SpillSimulator.O_BRIKKE, brett.brikkeVed(2, 0));
    assertEquals("Rute 0,1", SpillSimulator.TOM_RUTE, brett.brikkeVed(0, 1));
  }

  public void testStabling() {
    Spillebrett brett = SpillSimulator.byggSpillebrett("33");
    /* De tre nederste radene skal n se slik ut:

       2 | . . . . . . . |
       1 | . . . O . . . |
       0 | . . . X . . . |
         -----------------
        /  0 1 2 3 4 5 6  \
    */
    assertEquals("Frste sluppet", SpillSimulator.X_BRIKKE, brett.brikkeVed(3, 0));
    assertEquals("Andre sluppet", SpillSimulator.O_BRIKKE, brett.brikkeVed(3, 1));
    assertEquals("Kun to stablet", SpillSimulator.TOM_RUTE, brett.brikkeVed(3, 2));
  }

  public void testFullKolonne() {
    Spillebrett brett = SpillSimulator.byggSpillebrett("");
    int kolonne = 3;
    for (int i = 0; i < Spillebrett.RAD_ANTALL; ++i) {
      assertFalse(brett.erKolonneFull(kolonne));
      brett.slippBrikkeNedKolonne(brett.nesteBrikke(), kolonne);
    }
    assertTrue(brett.erKolonneFull(kolonne));
  }

  public void testErGyldigKolonneValg() {
    Spillebrett brett = SpillSimulator.byggSpillebrett("111111");
    assertFalse(brett.erGyldigKolonneValg( -1));
    assertTrue(brett.erGyldigKolonneValg(0));
    assertFalse(brett.erGyldigKolonneValg(1));
    assertTrue(brett.erGyldigKolonneValg(6));
    assertFalse(brett.erGyldigKolonneValg(7));
  }

  public void testEnkelSpillStatus() {
    Spillebrett start = SpillSimulator.byggSpillebrett("");
    assertFalse(start.erSpillFerdig());
    Spillebrett underveis = SpillSimulator.byggSpillebrett("340222244");
    assertFalse(underveis.erSpillFerdig());
    Spillebrett sisteTrekk = SpillSimulator.byggSpillebrett(
        "00000011111122222243333334444455555566666");
    assertFalse(sisteTrekk.erSpillFerdig());
    Spillebrett ingenVinner = SpillSimulator.byggSpillebrett(
        "000000111111222222433333344444555555666666");
    assertTrue(ingenVinner.erSpillFerdig());
  }

  public void testVinnerStatus() {
    Spillebrett xVinnerVertikal = SpillSimulator.byggSpillebrett(
        "2324252");
    assertTrue(xVinnerVertikal.erSpillFerdig());
    Spillebrett oVinnerHorisontal = SpillSimulator.byggSpillebrett(
        "06142335");
    assertTrue(oVinnerHorisontal.erSpillFerdig());
    Spillebrett xVinnerDiagonal = SpillSimulator.byggSpillebrett(
          "332401133414455556110666004050633");
    assertTrue(xVinnerDiagonal.erSpillFerdig());
  }
}
