// Program 8.12 i boken.
// Fil: Spillebrett.java
class Retning {
  int kolonneSkritt;
  int radSkritt;

  public static final Retning OPP = new Retning(0, 1);
  public static final Retning VENSTRE = new Retning(1, 0);
  public static final Retning DIAGONAL1 = new Retning(1, 1);
  public static final Retning DIAGONAL2 = new Retning(1, -1);
  public static final Retning[] RETNINGER = {OPP, VENSTRE, DIAGONAL1, DIAGONAL2};

  Retning(int dx, int dy) {
    kolonneSkritt = dx;
    radSkritt = dy;
  }
  Retning motsatt() {
    return new Retning( -kolonneSkritt, -radSkritt);
  }
}

public class Spillebrett {
  public static final int INGEN_LEDIG_RAD = -1;
  public static final char TOM_RUTE = '\u0000';
  public static final int RAD_ANTALL = 6;
  public static final int KOLONNE_ANTALL = 7;
  public static final int RUTE_ANTALL = RAD_ANTALL * KOLONNE_ANTALL;

  char[][] ruteInnhold = new char[KOLONNE_ANTALL][RAD_ANTALL];
  int trekkTeller;
  char vinner;
  ISpiller[] spillere;

  Spillebrett() {
    assert ruteInnhold[0][0] == TOM_RUTE;
  }

  void settSpillere(ISpiller[] begge_spillere) {
    assert begge_spillere.length == 2;
    spillere = begge_spillere;
  }

  char brikkeVed(int kolonne, int rad) {
    return ruteInnhold[kolonne][rad];
  }

  int finnNedersteLedigeRad(int kolonne) {
    char[] kolonneInnhold = ruteInnhold[kolonne];
    for (int rad = 0; rad < kolonneInnhold.length; ++rad) {
      if (kolonneInnhold[rad] == TOM_RUTE)
        return rad;
    }
    return INGEN_LEDIG_RAD;
  }

  boolean harVinner() {
    return vinner != '\u0000';
  }

  boolean erSpillFerdig() {
    return (trekkTeller >= RUTE_ANTALL) || harVinner();
  }

  void slippBrikkeNedKolonne(char brikke, int kolonne) {
    assert !erSpillFerdig();
    int rad = finnNedersteLedigeRad(kolonne);
    ruteInnhold[kolonne][rad] = brikke;
    if (lengsteSekvensSomInneholderRute(kolonne, rad) >= 4)
      vinner = brikke;
    ++trekkTeller;
  }

  int lengsteSekvensSomInneholderRute(int kolonne, int rad) {
    int strsteLengde = 1;
    // Let etter sekvenser av like brikker i alle retninger.
    for (Retning retning : Retning.RETNINGER) {
      int antallBrikkerPLinje = 1 + antallLikeRuterIRetning(kolonne, rad, retning)
          + antallLikeRuterIRetning(kolonne, rad, retning.motsatt());
      strsteLengde = Math.max(strsteLengde, antallBrikkerPLinje);
    }
    return strsteLengde;
  }

  int antallLikeRuterIRetning(int kolonne, int rad, Retning retning) {
    char brikke = brikkeVed(kolonne, rad);
    int antall = 0;
    while (true) {
      kolonne += retning.kolonneSkritt;
      rad += retning.radSkritt;
      if (!posisjonInneholderBrikke(kolonne, rad, brikke))
        break;
      ++antall;
    }
    return antall;
  }

  boolean gyldigKolonneIndeks(int kolonne) {
    return (kolonne >= 0) && (kolonne < KOLONNE_ANTALL);
  }

  boolean gyldigRadIndeks(int rad) {
    return (rad >= 0) && (rad < RAD_ANTALL);
  }

  boolean posisjonInneholderBrikke(int kolonne, int rad, char brikke) {
    return gyldigKolonneIndeks(kolonne)
        && gyldigRadIndeks(rad)
        && (brikkeVed(kolonne, rad) == brikke);
  }

  boolean erKolonneFull(int kolonne) {
    char[] kolonneInnhold = ruteInnhold[kolonne];
    int indeksForversteRute = kolonneInnhold.length - 1;
    char versteRute = kolonneInnhold[indeksForversteRute];
    return versteRute != TOM_RUTE;
  }

  boolean erGyldigKolonneValg(int kolonne) {
    return gyldigKolonneIndeks(kolonne)
        && !erKolonneFull(kolonne);
  }

  char nesteBrikke() {
    boolean erPartallstrekk = trekkTeller % 2 == 0;
    return erPartallstrekk ? 'X' : 'O';
  }

  ISpiller nesteSpiller() {
    return spillere[trekkTeller % 2];
  }
}


